import { Configuration } from "./Configuration";
import { DerivedKeyInfo, EncryptionType, PackedEncryptedContent } from "./constructs";
/**
 * Encryption session
 */
export declare class Session extends Configuration {
    /**
     * Decrypt some text
     * @param text The text to decrypt
     * @param password The password to use for decryption
     * @returns Decrypted text
     * @memberof Session
     */
    decrypt(text: string, password: string): Promise<string>;
    /**
     * Encrypt some text
     * @param text The text to encrypt
     * @param password The password to use for encryption
     * @returns A promise that resolves with encrypted text
     * @memberof Session
     */
    encrypt(text: string, password: string): Promise<PackedEncryptedContent>;
    /**
     * Derive a key using the current configuration
     * @param password The password
     * @param salt The salt
     * @param rounds Key derivation rounds
     * @param encryptionMethod Encryption method
     * @returns Derived key information
     * @protected
     * @memberof Session
     */
    _deriveKey(password: string, salt: string, rounds?: number, encryptionMethod?: EncryptionType): Promise<DerivedKeyInfo>;
    /**
     * Derive a new key using the current configuration
     * @param password The password
     * @returns Derived key information
     * @protected
     * @memberof Session
     */
    _deriveNewKey(password: string): Promise<DerivedKeyInfo>;
}
